#ifndef CSTATE_HH_
#define CSTATE_HH_


#include "State.hh"
#include "biu/Alphabet.hh"
#include <vector>

namespace ell
{	
		/**
		 * An object of the class CState represents a state in the energy
		 * landscape in a compressed representation.
		 * It can be transformed in and can be derived from a State object.
		 * 
		 * @author Martin Mann, Sebastian Will, Andreas Richter
		 */
	class State;

	class CState
	{
	protected:
			/*! Returns the compressed representation of the passed
			 *  sequence with the underlying alphabet alph.
			 */
		virtual std::vector<unsigned char> compressSeq(
				  const biu::Alphabet::Sequence& s, const biu::Alphabet* alph) const;

			/*! Returns the string representation of the passed
			 *  compressed sequence with the underlying alphabet alph.
			 *  @param cs the compressed representation of a state
			 *  @param alph the alphabet this compression is based on
			 *  @param seqLength the length of the uncompressed sequence
			 */
		virtual std::string uncompressSeq(const std::vector<unsigned char>& cs,
				  const biu::Alphabet* alph, size_t seqLength) const;

	public:
		CState()
		{}
		
		virtual ~CState()
		{}

		virtual bool			operator== (const CState& cState2) const = 0;
		virtual bool			operator!= (const CState& cState2) const = 0;

			/*! Returns a pointer to a clone of the current CState. */
		virtual CState*			clone() const = 0;

			/*! Returns a specific std::string representation of this
			 *  CState.
			 */
		virtual std::string	getCStateStringRepresentation() const = 0;

			/*! Returns a specific std::string representation of the
			 *  uncompressed representation of this state.
			 */
		virtual std::string	getUncomprStateStringRepresentation() const = 0;

			/*! Returns a pointer to a clone of the uncompressed state
				 representation of the current compressed state. */
		virtual State*			uncompressedClone() const = 0;

			/*! Fills the passed state with the uncompressed representation
			 *  of the current compressed state. */
		virtual void			uncompress(State& state) const = 0;

			/*! Returns a clone of the read CState from binary input
			 *  file stream. */
		virtual CState*		readBinaryInputClone(std::ifstream& source)
								 const = 0;

			/*! Writes CState binary to an output file stream. */
		virtual void			writeBinaryOutput(std::ofstream& destination)
								 const = 0;
	};

}

#endif /*CSTATE_HH_*/
