#include <ell/S_LP_PivotM.hh>
#include <cmath>	// for getFitness()
#include <ell/CS_LP_PivotM.hh>
#include <biu/assertbiu.hh>

namespace ell
{

	const size_t S_LP_PivotM::INVALID_INDEX =
		std::numeric_limits<size_t>::max();
	const size_t 
		S_LP_PivotM::RandomNeighborList::ItState::SWITCH_MODE_THRESHOLD = 10;
		
//////////////////////////////////////////////////////////
// S_LP_PivotM
//////////////////////////////////////////////////////////

	S_LP_PivotM::S_LP_PivotM(
	 const biu::LatticeModel* lattice, const biu::ContactEnergyFunction* energy,
	 const std::string& seq, const std::string& moveString,
	 const bool isAbsoluteMove)
	 : latProt(biu::LatticeProtein(lattice, energy, seq, moveString,
							   isAbsoluteMove)),
	   pivotMoveSet(lattice)
	{
		assertbiu(latProt.isValid(),
		 "S_LP_PivotM has to be a valid state.");
	}

	S_LP_PivotM::S_LP_PivotM(
				const S_LP_PivotM& latProtPMState2)
	 : latProt(latProtPMState2.latProt),
	   pivotMoveSet(latProtPMState2.latProt.getLatticeModel()) {
	}

	S_LP_PivotM::~S_LP_PivotM() {
	}

	S_LP_PivotM::PivotMove
	S_LP_PivotM::firstPivotMove() const {
			// first move has to be applied at position 1
		return nextPivotMove(PivotMove(0, pivotMoveSet.getMoveNumber()));
	}
		
	S_LP_PivotM::PivotMove
	S_LP_PivotM::nextPivotMove(const PivotMove lastPivotMove) const {
		if (lastPivotMove.second+1 < pivotMoveSet.getMoveNumber()) {
			return PivotMove(lastPivotMove.first, lastPivotMove.second+1);
		}
			// if the move index reached the number of available moves,
			// increase the move position by one
		else {
			if (lastPivotMove.first+1 < latProt.getStructureRef().size()) {
				return PivotMove(lastPivotMove.first+1, 0);
			}
			// no more pivot moves possible
			else {
				return PivotMove(0,INVALID_INDEX);
			}
		}
	}

	bool	
	S_LP_PivotM::operator== (const State& state2) const {
		return latProt == dynamic_cast<
			const S_LP_PivotM&>(state2).latProt;
	}

	bool
	S_LP_PivotM::operator!= (const State& state2) const {
		return latProt != dynamic_cast<
			const S_LP_PivotM&>(state2).latProt;
	}

	int	
	S_LP_PivotM::getFitness() const {
			// rounds energy and casts it an int-value, then negate it
		double energy = latProt.getEnergy();
		if (energy >= 0)
			return - static_cast<int>(floor(energy + 0.5));
		else
			return - static_cast<int>(ceil(energy - 0.5));
	}

	double	
	S_LP_PivotM::getEnergy(void) const {
		return latProt.getEnergy();
	}

	unsigned int	
	S_LP_PivotM::getMinimalDistance(
		 const State& _state2) const {
		S_LP_PivotM state2 = 
		 dynamic_cast<const S_LP_PivotM&>(_state2);

		const biu::Structure& state1Struct = latProt.getStructureRef();
		const biu::Structure& state2Struct = state2.latProt.getStructureRef();

		assertbiu(state2Struct.size() == state1Struct.size(),
		 "Both lattice proteins have to have the same length.");
			
		unsigned int distance = 0;
		for (size_t i = 0; i < state1Struct.size(); i++){
			if (state1Struct[i] != state2Struct[i]) {
				distance += 1;
			}
		}
		return distance;
	}

	S_LP_PivotM* 
	S_LP_PivotM::clone() const {
		return new S_LP_PivotM(*this);
	}
	
	std::string 
	S_LP_PivotM::getStateStringRepresentation()
		     const{
		return latProt.getStringRepresentation();
	}

	CState*
	S_LP_PivotM::compressedClone() const {
		return new CS_LP_PivotM(
		 latProt.getLatticeModel(), latProt.getContactEnergyFunction(),
		 latProt.getSequence(), latProt.getStructureRef());
	}

	void
	S_LP_PivotM::compress(CState& cState) const {
		dynamic_cast<CS_LP_PivotM&>(cState)
		 = CS_LP_PivotM(
			latProt.getLatticeModel(), latProt.getContactEnergyFunction(),
			latProt.getSequence(), latProt.getStructureRef());
	}

	State::NeighborListPtr 
	S_LP_PivotM::getNeighborList() const {
		return NeighborListPtr(new NeighborList(this));
	}

	State::NeighborListPtr
	S_LP_PivotM::getRandomNeighborList() const {
		return NeighborListPtr(new RandomNeighborList(this));
	}

	// NeighborList
	S_LP_PivotM::NeighborList::NeighborList(
	 			const S_LP_PivotM* _origin) 
	 : origin(_origin) 
	{
	}

	S_LP_PivotM::NeighborList::~NeighborList() {
	}

	State* 
	S_LP_PivotM::NeighborList::first(
		      State::NeighborList::ItState** itstate) const {
		if (*itstate != NULL) delete *itstate;

		ItState* myitstate = new ItState();
		*itstate = myitstate;

		S_LP_PivotM* latProtState = NULL;

		PivotMove firstMove = origin->firstPivotMove();

		if (firstMove != PivotMove(0,INVALID_INDEX)) {
			latProtState = new S_LP_PivotM(*origin);
			latProtState->pivotMoveSet.applyMoveInPlace(&latProtState->latProt,
			 firstMove.first, firstMove.second);
			myitstate->lastMove = firstMove;
				// if applied pivot move generates an invalid neighbor,
				// call next(..) to iterate neighbors until a VALID one is
				// found which will be returned
			if (!(latProtState->latProt.isValid())) {
				return next(*itstate, latProtState);
			}
		}

		return latProtState;
	}

	State* 
	S_LP_PivotM::NeighborList::next(
		      State::NeighborList::ItState* itstate, State* elem) const {
		assertbiu(elem != NULL, "Elem is not allowed to be NULL");

		ItState* myitstate = static_cast<ItState*>(itstate);

			// find next move
		PivotMove nextMove = origin->nextPivotMove(myitstate->lastMove);
			// no more moves possible
		if (nextMove == PivotMove(0,INVALID_INDEX)) {
			return NULL;
		}
		else {
			S_LP_PivotM* latProtState =
			 dynamic_cast<S_LP_PivotM*>(elem);
				// set latProtState to the state of origin of the neighbors
			latProtState->latProt = origin->latProt;
				// perform next move
			latProtState->pivotMoveSet.applyMoveInPlace(&latProtState->latProt,
			 nextMove.first, nextMove.second);
			myitstate->lastMove = nextMove;
				// if applied pivot move generates an invalid neighbor,
				// call next(..) to iterate neighbors until a VALID one is
				// found which will be returned
			if (!(latProtState->latProt.isValid())) {
				return next(itstate, latProtState);
			}
			return latProtState;
		}
	}

	// RandomNeighborList
	S_LP_PivotM::RandomNeighborList::RandomNeighborList(
	 		const S_LP_PivotM* _origin)
	 : origin(_origin) 
	{
	}

	S_LP_PivotM::RandomNeighborList::~RandomNeighborList() {
	}

	void 
	S_LP_PivotM::RandomNeighborList::switchMode(
			ItState* itstate) const {
		ItState* myitstate = static_cast<ItState*>(itstate);

		PivotMove nextMove = origin->firstPivotMove();

		// loop while moves exist
		while (nextMove != PivotMove(0,INVALID_INDEX)) {
			// add all unchosen moves to unchosenMoves
			if (myitstate->chosenMoves.find(nextMove) == 
			     myitstate->chosenMoves.end()) {
				myitstate->unchosenMoves.push_back(nextMove);
			}
			nextMove = origin->nextPivotMove(nextMove);
		}
	}

	State* 
	S_LP_PivotM::RandomNeighborList::first(
		      State::NeighborList::ItState** itstate) const {
		if (*itstate != NULL) delete *itstate;

		*itstate = new ItState();
		S_LP_PivotM* latProtState = 
		 new S_LP_PivotM(*origin);

		return next(*itstate, latProtState);
	}

	State* 
	S_LP_PivotM::RandomNeighborList::next(
		      State::NeighborList::ItState* itstate, State* elem) const {
		assertbiu(elem != NULL, "Elem is not allowed to be NULL");

		ItState* myitstate = static_cast<ItState*>(itstate);
		S_LP_PivotM* latProtState =
		 dynamic_cast<S_LP_PivotM*>(elem);

		PivotMove nextMove;

			// try to find a valid move at random
		if (myitstate->switchModeValue > 0) {
				// position and index of the next pivot move
			size_t pos, moveIndex;
			bool foundValidMove = false;

			while (!foundValidMove) {
					// move position has to be in [1, structure length)
				pos = (random() % (origin->latProt.getStructureRef().size()-1))+1;
					// moveIndex has to be in [0, number of available moves)
				moveIndex = random() % origin->pivotMoveSet.getMoveNumber();

				nextMove = PivotMove(pos, moveIndex);

					// move has already been chosen
				if (!myitstate->chosenMoves.insert(nextMove).second) {
					continue;
				}
					// move hasn't been chosen yet
				else {
						// set latProtState to the state of origin of the
						// neighbors
					latProtState->latProt = origin->latProt;
					latProtState->pivotMoveSet.applyMoveInPlace(&latProtState->latProt,
					 nextMove.first, nextMove.second);
						// if applied pivot move generates an invalid neighbor,
						// iterate neighbors until a VALID one is found which
						// will be returned
					foundValidMove = latProtState->latProt.isValid();
					if (foundValidMove) {
						myitstate->switchModeValue--;
					}
				}	
			}
				// enough valid neighbors seen, switching mode to complete
				// valid neighbor enumeration
			if (myitstate->switchModeValue == 0) {
				switchMode(myitstate);
			}
		}
			// chose one of the precalculated valid moves 
		else {
			size_t n = myitstate->unchosenMoves.size();
			if (n == 0) {
				return NULL;
			}
			else {

// @TODO : Rekursion rausnehmen !!!

				size_t i = random() % n;
					// set latProtState to the state of origin of the neighbors
				latProtState->latProt = origin->latProt;
				nextMove = myitstate->unchosenMoves[i];
				latProtState->pivotMoveSet.applyMoveInPlace(
									&latProtState->latProt,
									nextMove.first, nextMove.second);
				myitstate->unchosenMoves[i] = myitstate->unchosenMoves[n-1];
				myitstate->unchosenMoves.resize(n-1);
					// if applied pivot move generates an invalid neighbor,
					// call next(..) to iterate neighbors until a VALID one is
					// found which will be returned
				if (!(latProtState->latProt.isValid())) {
					return next(itstate, latProtState);
				}
			}
		}
		return latProtState;
	}
	
	State* 
	S_LP_PivotM::getRandomNeighbor(State* inPlaceNeigh) const {
		S_LP_PivotM* neigh = NULL;
			// copy this state
		if (inPlaceNeigh != NULL) {
			neigh = dynamic_cast<S_LP_PivotM*>(inPlaceNeigh);
			assertbiu(neigh != NULL, "inPlaceNeigh is no S_LP_PivotM"); 
			(*neigh) = (*this); 
		} else {
			neigh = new S_LP_PivotM(*this);
		}

			// position and index of the next pivot move
		size_t pos, moveIndex;

		while (true) {
				// move position has to be in [1, structure length)
			pos = (random() % (latProt.getStructureRef().size()-1))+1;
				// moveIndex has to be in [0, number of available moves)
			moveIndex = random() % pivotMoveSet.getMoveNumber();

				// apply change
			pivotMoveSet.applyMoveInPlace(&neigh->latProt, pos, moveIndex);
				// if applied pivot move generates an invalid neighbor,
				// iterate neighbors until a VALID one is found which
				// will be returned
			if(neigh->latProt.isValid()) {
				return neigh;
			} else {
				// restore lattice protein
				pivotMoveSet.undoLastMove(&neigh->latProt);
			}
		}		
		return NULL;
	}


} // namespace ell
