#ifndef LATTICEPROTEINPIVOTMOVESTRUCTCUBNORMSTATE_HH_
#define LATTICEPROTEINPIVOTMOVESTRUCTCUBNORMSTATE_HH_


#include "S_LP_PivotM.hh"
#include "biu/LatticeDescriptorCUB.hh"

namespace ell
{
		/**
		 * This class generates neighbors of lattice proteins in the
		 * energy landscape by applying pivot moves.
		 * All lattice proteins of this class have a common sequence
		 * and contact energy function.
		 * The lattice model is set to cubic lattice and two proteins
		 * are equal iff any of their symmetric representations are
		 * equal.
		 *
		 * @author Martin Mann, Sebastian Will, Andreas Richter
		 */
	class S_LP_PivotM_StructCUBNorm :
	 public S_LP_PivotM
	{
	protected:	
		class NeighborList : public S_LP_PivotM::NeighborList
		{
		protected:
				//! the state of origin of the neighbors
			const S_LP_PivotM_StructCUBNorm* const origin;

		public:
			NeighborList(const S_LP_PivotM_StructCUBNorm*
								_origin);

				/*! Returns a pointer to the first element of the
				 *  virtual list of neighbors of the state of origin.
				 *  Returns NULL if no neighbors exist.
				 */
			virtual State* first(State::NeighborList::ItState** itstate) const;
						 
				/*! Returns a pointer to the next element of the
				 *  virtual list of neighbors of the state of origin.
				 *  Returns NULL if no more neighbors exist.
				 */
			virtual State* next(	State::NeighborList::ItState* itstate, 
									State* elem) const;
		};

		class RandomNeighborList : public S_LP_PivotM::
											RandomNeighborList
		{
		protected:
				//! the state of origin of the neighbors
			const S_LP_PivotM_StructCUBNorm* const origin;

		public:
			RandomNeighborList(const S_LP_PivotM_StructCUBNorm*
									  _origin);

				/*! Returns a pointer to a random element of the
				 *  virtual list of neighbors of the state of origin.
				 *  Returns NULL if no neighbors exist.
				 */
			virtual State* first(State::NeighborList::ItState** itstate) const;
						 
				/*! Returns a pointer to the next random element of the
				 *  virtual list of neighbors of the state of origin.
				 *  Returns NULL if no more neighbors exist.
				 */
			virtual State* next(	State::NeighborList::ItState* itstate, 
									State* elem) const;
		};

			/*! common cubic lattice property handler */
		static const biu::LatticeDescriptorCUB commonCUBLd;

			/*! common cubic lattice model */
		static const biu::LatticeModel		commonCUBLattice;

			/*! common contact energy function */
		static const biu::ContactEnergyFunction*	commonEnergy;	

			//! common sequence string
		static std::string				commonSeqStr;

			/*! Returns the lexicographically smallest relative move string
			 *	 of this state. The relative moves have the lexicographical
			 *  FLDRU. */
		std::string normalize() const;

			/*! Swaps the character in the passed string s in the way that 'L'
			 *  becomes l, 'D' becomes d, 'R' becomes r and 'U' becomes u
			 *  beginning at position 1.
			 */
		void swapCharsInString(std::string& s, char l, char d, char r, char u)
			  const;

	public:
			 /*! The common sequence and contact energy function
			  *  have to be set before calling the constructor.
			  */
		S_LP_PivotM_StructCUBNorm(
		 const std::string& moveString, const bool isAbsoluteMove);

		S_LP_PivotM_StructCUBNorm( 
		 const S_LP_PivotM_StructCUBNorm& latProtCUBNormState2);
						
		virtual ~S_LP_PivotM_StructCUBNorm();

			/*! Sets common contact energy function.
			 *
			 *  Has to be called before initialisation of an
			 *  S_LP_PivotM_StructCUBNorm object and
			 *  has to be called only once.
			 */
		static void		setCommonContactEnergyFunction(
						 const biu::ContactEnergyFunction* energy);

			/*! Sets common sequence string.
			 *
			 *  Has to be called before initialisation of an
			 *  S_LP_PivotM_StructCUBNorm object and
			 *  has to be called only once.
			 */
		static void		setCommonSequenceString(const std::string& seqStr);

			/*! Returns the common cubic lattice model.
			 */
		static const biu::LatticeModel*			getCommonCUBLatticeModel() {
			return &commonCUBLattice;
		}

			/*! Returns the contact energy function.
			 */
		static const biu::ContactEnergyFunction*	getCommonContactEnergyFunction() {
			return commonEnergy;
		}

			/*! Returns the common sequence string.
			 */
		static std::string					getCommonSequenceString() {
			return commonSeqStr;
		}

	// abstract functions (State)
		 	/*! Two states are equal iff any of their symmetric
			 *  representations (i.e. the state's lexicographically smallest
			 *  relative move strings) are equal.
		 	 */
		virtual bool	operator== (const State& state2) const;
			 /*! Two states are unequal iff all of their symmetric
			  *  representations (i.e. the state's lexicographically smallest
			  *  relative move strings) are unequal.
			  */
		virtual bool	operator!= (const State& state2) const;

			/*! Returns a pointer to a clone of the current state. */
		virtual S_LP_PivotM_StructCUBNorm*	clone() const;
	
			/*! Returns a pointer to a clone of the compressed NORMALIZED
			 *  current state.
			 */
		virtual CState* compressedClone() const;

			/*! Fills the passed CState with the compressed NORMALIZED
			 *  current state.
			 */
		virtual void compress(CState& cState) const;

		virtual NeighborListPtr getNeighborList() const;
		
		virtual NeighborListPtr getRandomNeighborList() const;

		virtual State* getRandomNeighbor(State* inPlaceNeigh = NULL) const;

	};

} // namespace ell

#endif /*LATTICEPROTEINPIVOTMOVESTRUCTCUBNORMSTATE_HH_*/
