#ifndef STATE_H_
#define STATE_H_


#include <string>
#include <biu/VirtualList.hh>
#include "CState.hh"

	/** 
	 * The namespace ell is a shortcut for the Energy Landscape Library, 
	 * generated on the chair of Bioinformatics at the University of Freiburg.
	 * http://www.bioinf.uni-freiburg.de/
	 * 
	 * It covers a set of classes and methods to model and handle arbitrary 
	 * types of energy landscapes using the abstract interface class State 
	 * or derived subclasses. 
	 * 
	 * @author Martin Mann, Sebastian Will, Andreas Richter
	 */
namespace ell
{	
		/**
		 * An object of the class State represents a state in the energy
		 * landscape.
		 * It handles its fitness and neighborhood concerning the landscape.
		 * 
		 * A State is always valid in the subclass.
		 * 
		 * @author Martin Mann, Sebastian Will, Andreas Richter
		 */
	class CState;

	class State
	{
	public:
		// TYPEDEFs
	
			//! Virtual list of neighbor states which can be accessed
			//! by a STL-like iterator.
		typedef biu::VirtualList<State> NeighborList;
		
			//! An autopointer to a NeighborList
		typedef std::auto_ptr<NeighborList> NeighborListPtr;

		State()
		{}
		
		virtual ~State()
		{}
		
		virtual bool			operator== (const State& state2) const = 0;
		virtual bool			operator!= (const State& state2) const = 0;

			/*! Returns the state specific fitness. */
		virtual int			getFitness() const = 0;
		
			/*! Returns the state specific energy. */
		virtual double		getEnergy() const = 0;
		
			/*! Returns the minimal number of steps via valid
			 *  neighbored states from this to another valid State.
			 *  @param state2 the State to reach
			 */
		virtual unsigned int	getMinimalDistance(const State& state2) const = 0;
		
			/*! Returns a pointer to a clone of the current state. */
		virtual State*			clone() const = 0;
		
			/*! Returns a specific std::string representation of this
			 *  State.
			 */
		virtual std::string	getStateStringRepresentation() const = 0;

			/*! Returns a pointer to a clone of the compressed state
				 representation of the current uncompressed state. */
		virtual CState*		compressedClone() const = 0;

			/*! Fills the passed compressed state with the compressed
			 *  representation of the current uncompressed state. */
		virtual void			compress(CState& cState) const = 0;

		// neighborhood
			/*! Returns a virtual list of all VALID neighbored states in
			 *  the energy landscape in a specific order.
			 */
		virtual NeighborListPtr getNeighborList() const = 0;

			/*! Returns a virtual list of all VALID neighbored states in
			 *  the energy landscape in a random order.
			 *  If a state is given, this is changed to a neighbor.
			 */
		virtual NeighborListPtr getRandomNeighborList() const = 0;

			/*! Returns a VALID random neighbored state.
			 *  If a state is given, this is changed to a neighbor.
			 */
		virtual State* getRandomNeighbor(State* inPlaceNeigh = NULL) const = 0;
	};

}

#endif /*STATE_H_*/
