
#include "ell/walkMC.hh"
#include "ell/State.hh"
#include "ell/WalkCollector.hh"
#include "ell/WC_Listing.hh"
#include "ell/StateAcceptor.hh"
#include <iostream>
#include <string>
#include <vector>
#include <stdlib.h>
#include <math.h>


using namespace ell;

/*!
 * This is a dummy implementation of a State class including inner classes
 * NeighborList and ItState.
 * It generates neighbors starting with relative energy +3 compared
 * to initial State. Each following successor's energy is decreased by one.
 * There is an energy limit, after which no more successors are produced to 
 * guarantee determination of walkMC.
 * 
 * @author Daniel Maticzka
 */
class DummyState : public State
{
public:
	class NeighborList : public State::NeighborList 
	{
	private:
		std::string id;
		double origin;
		double maxEnergy;
	public:
		class ItState : public State::NeighborList::ItState {
		public:
			int pos;
			
			ItState():pos(0) {}
			
			ItState(const ItState &itstate) {pos=itstate.pos;}
			
			virtual ItState * clone() const {
				return new ItState(*this);
			}
		};
    
		NeighborList(std::string _id, double _origin, double _maxEnergy) 
		{
			id = _id;
			// first successor starts with relative energy +3
			origin = _origin + 4;
			maxEnergy = _maxEnergy;
		}
    
		virtual ~NeighborList() {}
  
		virtual State*
		next(State::NeighborList::ItState *itstate, State* elem) const {
			((DummyState*)elem)->applyMove();
			if (elem->getEnergy() > maxEnergy) return NULL;
			return elem;
		}

		virtual State* 
		first(State::NeighborList::ItState** itstate) const {
			DummyState* s = new DummyState(id, origin, maxEnergy);
			s->applyMove();
			return s;
		}
	};
	
	std::string id;
	double energy;
	double maxEnergy;

	DummyState(std::string _id, double _energy, double _maxEnergy) 
		{ id = _id; energy = _energy; maxEnergy = _maxEnergy;} 
	~DummyState() {}
	
	bool operator== (const State& state2) const { return true; }
	bool operator!= (const State& state2) const { return true; }

	int	getFitness() const { return 0; }
	double getEnergy() const { return energy; }
	unsigned int getMinimalDistance(const State& state2) const { return 0; }
	State* clone() const { return new DummyState(id, energy, maxEnergy); }
	std::string	getStateStringRepresentation() const { return id; }
	CState*	compressedClone() const { return NULL; }
	void compress(CState& cState) const {}
	
	NeighborListPtr getNeighborList() const 
	{ 
		return NeighborListPtr(new NeighborList(id,energy,maxEnergy));
	}
	
	NeighborListPtr getRandomNeighborList() const {
		return NeighborListPtr(new NeighborList(id,energy,maxEnergy));
	}
	
	State* getRandomNeighbor(State* inPlaceNeigh = NULL) const { return NULL; }
	
	DummyState& applyMove() {
		// decrease energy by one
		energy -= 1;
		return *this;
	}
};

int main(int argc, char **argv) {
	
	// static initialization of rng 
	srand(0);
	
	std::cout	<<"========== BEGIN OF TEST ==================\n"
				<<" test function : walkMC\n"
				<<"===========================================" <<std::endl;
	
	std::cout	<< "==> testing RNG" << std::endl;
	for (int i=1; i<=10; i++){
	std::cout 	<< i << ": " << (((double) rand()) / ((double) RAND_MAX)) << std::endl;
	}
	
	std::cout	<< "=> evaluation of used metropolis criterion:" << std::endl;
	std::cout	<< "exp((-1)* -5 ) = " << exp((-1)* -5 ) << std::endl;
	std::cout	<< "exp((-1)* -4 ) = " << exp((-1)* -4 ) << std::endl;
	std::cout	<< "exp((-1)* -3 ) = " << exp((-1)* -3 ) << std::endl;
	std::cout	<< "exp((-1)* -2 ) = " << exp((-1)* -2 ) << std::endl;
	std::cout	<< "exp((-1)* -1 ) = " << exp((-1)* -1 ) << std::endl;
	std::cout	<< "exp((-1)* 0 ) = " << exp((-1)* 0 ) << std::endl;
	std::cout	<< "exp((-1)* 1 ) = " << exp((-1)* 1 ) << std::endl;
	std::cout	<< "exp((-1)* 2 ) = " << exp((-1)* 2 ) << std::endl;
	std::cout	<< "exp((-1)* 3 ) = " << exp((-1)* 3 ) << std::endl;
	std::cout	<< "exp((-1)* 4 ) = " << exp((-1)* 4 ) << std::endl;
	std::cout	<< "exp((-1)* 5 ) = " << exp((-1)* 5 ) << std::endl;


	std::cout	<< "=> creating SA_Metropolis" << std::endl;
	WalkCollector* wc_pre = new WC_Listing();
	State& state_pre = *(new DummyState("WC_State", 20,20));
	wc_pre->setAccepted(state_pre);
	State& state_succ = *(new DummyState("state 1", 21,21));
	StateAcceptor* sa = new SA_Metropolis(1);
	for (int i=1; i<=10; i++){
		std::cout 	<< "->accept(old energy: 20, successor energy: 21): " 
					<< sa->accept(wc_pre, state_succ) 
					<< std::endl;
	}
	delete &state_pre;
	delete &state_succ;
	delete wc_pre;
	delete sa;
	
	std::cout	<< "=> creating DummyState" << std::endl;
	State* state = new DummyState("state 1", 20,28);
	std::cout	<< "->getEnergy = " << state->getEnergy() << std::endl;
	std::cout	<< "=> printing 5 successors:" << std::endl;
	State::NeighborListPtr neighbors = state->getNeighborList();
	State::NeighborList::Iterator it = neighbors->begin();
	for (int i=1; i<5; i++) {
		std::cout	<< i << ". successor: " << (*it).getEnergy() << std::endl;
		++it;
	}
	
	std::cout	<< "=> creating WalkCollector" << std::endl;
	WalkCollector* wc = new WC_Listing();
	std::cout	<< "=> calling walk" << std::endl;
	double beta = 1;
	wc = walkMC(state, wc, beta);
	
	std::cout 	<< "Number Of Accepted States: " 
					<< wc->getAcceptedCount()
					<< std::endl;
	
	WC_Listing* wch;
	wch = dynamic_cast<WC_Listing*>(wc);
	
	std::cout 	<< "Accepted States:" << std::endl;
	std::vector<State*> vec = wch->getAcceptedList();
	std::vector<State*>::iterator iter;
  	for( iter = vec.begin(); iter != vec.end(); iter++ ) {
    	std::cout 	<< "  * Energy = "
    				<< (*iter)->getEnergy() << std::endl;
  	}

  	delete state;
  	delete wc;
  	
  	std::cout	<<"============= END OF TEST =================\n" <<std::endl;
  	
	return 0; 
}
